﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Unitronics.ComDriver;
using System.Timers;
using Unitronics.ComDriver.Messages.DataRequest;
using Listener_Server_Example.Commands;
using System.Windows.Input;
using System.Diagnostics;
using System.Runtime.Remoting.Messaging;


namespace Listener_Server_Example.ViewModels
{
    public class ClientViewModel : ViewModelBase
    {
        private PLC plc;
        private string clientIP;
        private string plcModel;
        private string osVersion;
        private string plcName;
        private string sdw0;
        Timer timer;
        private DelegateCommand disconnectClientCommand;
        private DelegateCommand resetPlcCommand;
        private DelegateCommand stopPlcCommand;
        private DelegateCommand runPlcCommand;

        public event EventHandler OnConnectionClosed;

        public ClientViewModel(PLC plcFromListener)
        {
            plc = plcFromListener;
            (plc.PLCChannel as ListenerClient).OnConnectionClosed += new ListenerClient.ConnectionClosedDelegate(ClientViewModel_OnConnectionClosed);
            ClientIP = (plc.PLCChannel as ListenerClient).RemoteIP;
            PlcVersion version = plc.Version;
            PlcModel = version.OPLCModel;
            OsVersion = version.OSVersion + "(" + version.OSBuildNum + ")";

            Action action = delegate()
            {
                PlcName = plc.PlcName;
            };
            action.BeginInvoke(null, null);

            timer = new Timer(100);
            timer.Elapsed += new ElapsedEventHandler(timer_Elapsed);
            timer.Start();
        }

        void ClientViewModel_OnConnectionClosed(ListenerClient ethernetListener)
        {
            if (timer != null)
                timer.Stop();
            if (OnConnectionClosed != null)
                OnConnectionClosed(this, null);
        }

        public string ClientIP
        {
            get
            {
                return clientIP;
            }
            internal set
            {
                clientIP = value;
                OnPropertyChanged("ClientIP");
            }
        }

        public string PlcModel
        {
            get
            {
                return plcModel;
            }
            internal set
            {
                plcModel = value;
                OnPropertyChanged("PlcModel");
            }
        }

        public string OsVersion
        {
            get
            {
                return osVersion;
            }
            internal set
            {
                osVersion = value;
                OnPropertyChanged("OsVersion");
            }
        }

        public string PlcName
        {
            get
            {
                return plcName;
            }
            internal set
            {
                plcName = value;
                OnPropertyChanged("PlcName");
            }
        }

        public string Sdw0
        {
            get
            {
                return sdw0;
            }
            internal set
            {
                sdw0 = value;
                OnPropertyChanged("Sdw0");
            }
        }

        public ICommand DisconnectClientCommand
        {
            get
            {
                if (disconnectClientCommand == null)
                {
                    disconnectClientCommand = new DelegateCommand(disconnectClient);
                }
                return disconnectClientCommand;
            }
        }

        private void disconnectClient()
        {
            try
            {
                // When using a Listener Server, it is important to close the connection to the PLC
                // if it's not needed anymore. plc.Disconnect closes the connection only if there are no
                // pending messages in the queue (This is why it is important to abort the communication before disconnecting).

                // However, plc.Dispose will allow you to close the connection anyway (if other PLC objects use the same connection
                // which is the same ListenerClient) then they will become diposed as well.
                // Dispose method is only relevant to ListenerClient connections.
                plc.Dispose();
            }
            catch
            {
            }
        }

        public ICommand ResetPlcCommand
        {
            get
            {
                if (resetPlcCommand == null)
                {
                    resetPlcCommand = new DelegateCommand(resetPlc);
                }
                return resetPlcCommand;
            }
        }

        private void resetPlc()
        {
            try
            {
                // Reseting the PLC will cause the connection to be closed.
                // If the PLC is connected directly to the PC using a cable then the PC should get a disconnection event.
                // However, if there is a hub in the middle the PC will get the disconnection event only after it tries to communicate with the PLC.
                // In this example we read SDW 0 every 100ms which will cause the connection to be "tested" just like mentioned above.
                plc.Reset();
            }
            catch
            {
            }
        }

        public ICommand StopPlcCommand
        {
            get
            {
                if (stopPlcCommand == null)
                {
                    stopPlcCommand = new DelegateCommand(stopPlc);
                }
                return stopPlcCommand;
            }
        }

        private void stopPlc()
        {
            try
            {
                plc.Stop();
            }
            catch
            {
            }
        }

        public ICommand RunPlcCommand
        {
            get
            {
                if (runPlcCommand == null)
                {
                    runPlcCommand = new DelegateCommand(runPlc);
                }
                return runPlcCommand;
            }
        }

        private void runPlc()
        {
            try
            {
                plc.Run();
            }
            catch
            {
            }
        }

        void timer_Elapsed(object sender, ElapsedEventArgs e)
        {
            try
            {
                ReadWriteRequest[] rw = new ReadWriteRequest[1];
                rw[0] = new ReadOperands()
                {
                    NumberOfOperands = 1,
                    OperandType = OperandTypes.SDW,
                    StartAddress = 0,
                };

                plc.ReadWrite(ref rw);


                Sdw0 = ((rw[0].ResponseValues as object[])[0]).ToString();
            }
            catch (ComDriveExceptions cde)
            {
                if (cde.ErrorCode == ComDriveExceptions.ComDriveException.ObjectDisposed)
                {
                    plc.Dispose();
                }
            }
            catch (Exception ex)
            {
            }
        }
    }
}
